@@ -23,12 +23,22 @@ tscriptname=`basename $0 .sh`
 export PKG=`echo $tscriptname | sed -e 's/\-[^\-]*\-[^\-]*$//'`
 export PKG2=libpng10
 export PKG3=libpng10-devel
-export VER=`echo $tscriptname | sed -e 's/^[^\-]*\-//' -e 's/\-[^\-]*$//'`
-export REL=`echo $tscriptname | sed -e 's/^[^\-]*\-[^\-]*\-//'`
+export VER=`echo $tscriptname | sed -e "s/${PKG}\-//" -e 's/\-[^\-]*$//'`
+export REL=`echo $tscriptname | sed -e "s/${PKG}\-${VER}\-//"`
 export FULLPKG=${PKG}-${VER}-${REL}
 export FULLPKG2=${PKG2}-${VER}-${REL}
 export FULLPKG3=${PKG3}-${VER}-${REL}
-export src_orig_pkg_name=${PKG}-${VER}.tar.bz2
+
+# determine correct decompression option and tarball filename
+if [ -e ${PKG}-${VER}.tar.gz ] ; then
+  export opt_decomp=z
+  export src_orig_pkg_ext=gz
+elif [ -e ${PKG}-${VER}.tar.bz2 ] ; then
+  export opt_decomp=j
+  export src_orig_pkg_ext=bz2
+fi
+
+export src_orig_pkg_name=${PKG}-${VER}.tar.${src_orig_pkg_ext}
 export src_pkg_name=${FULLPKG}-src.tar.bz2
 export src_patch_name=${FULLPKG}.patch
 export bin_pkg_name=${FULLPKG}.tar.bz2
@@ -57,35 +67,41 @@ snoslash=`echo $sysconfdir | sed -e 's#^/##'`
 
 mkdirs() {
   (cd ${topdir} && \
+  rm -fr ${objdir} ${instdir} ${srcinstdir} && \
   mkdir -p ${objdir} && \
   mkdir -p ${instdir} && \
-  mkdir -p ${srcinstdir} )
-}
+  mkdir -p ${srcinstdir}
+)}
+
 prep() {
   (cd ${topdir} && \
-  tar xvjf ${src_orig_pkg} ; \
+  tar xv${opt_decomp}f ${src_orig_pkg} ; \
   cd ${srcdir} && \
   patch -p1 < ${src_patch} 
   && mkdirs && \
   if [ -f ${srcdir}/scripts/makefile.cygwin ] ; then \
     cp ${srcdir}/scripts/makefile.cygwin ${objdir}/Makefile ;\
-  fi ) 
-}
+  fi
+)}
+
 conf() {
   echo Configuration successful.
 }
 build() {
   (cd ${objdir} && \
-  make SRCDIR=${srcdir} prefix=${prefix} )
-}
+  make SRCDIR=${srcdir} prefix=${prefix}
+)}
+
 check() {
   (cd ${objdir} && \
-  make test SRCDIR=${srcdir} prefix=${prefix} | tee ${checkfile} 2>&1 )
-}
+  make test SRCDIR=${srcdir} prefix=${prefix} | tee ${checkfile} 2>&1
+)}
+
 clean() {
   (cd ${objdir} && \
-  make clean SRCDIR=${srcdir} prefix=${prefix} )
-}
+  make clean SRCDIR=${srcdir} prefix=${prefix}
+)}
+
 install() {
   (cd ${objdir} && \
   make install DESTDIR=${instdir} SRCDIR=${srcdir} prefix=${prefix} &&
@@ -97,8 +113,8 @@ install() {
       rm -f ${instdir}${fn} ; \
     fi ;\
   done && \
-  for dirn in ${prefix}/doc/${PKG}-${VER} ${prefix}/doc/${PKG3}-${VER} \
-    ${prefix}/doc/Cygwin ${sysconfdir}/postinstall ; do \
+  for dirn in ${prefix}/share/doc/${PKG}-${VER} ${prefix}/share/doc/${PKG3}-${VER} \
+    ${prefix}/share/doc/Cygwin ${sysconfdir}/postinstall ; do \
     if [ ! -d ${instdir}${dirn} ]; then \
       mkdir -p ${instdir}${dirn} ; \
     fi ;\
@@ -106,40 +122,42 @@ install() {
   templist=""; \
   for f in ${srcdir}/ANNOUNCE ${srcdir}/CHANGES ${srcdir}/INSTALL \
            ${srcdir}/KNOWNBUG ${srcdir}/LICENSE ${srcdir}/README \
-			  ${srcdir}/TODO ${srcdir}/Y2KINFO ${srcdir}/example.c ; do \
+           ${srcdir}/TODO ${srcdir}/Y2KINFO ${srcdir}/example.c ; do \
     if [ -f $f ] ; then \
       templist="$templist $f"; \
     fi ; \
   done && \
   if [ ! "x$templist" = "x" ]; then \
     /usr/bin/install -m 644 $templist \
-         ${instdir}${prefix}/doc/${PKG}-${VER} ;
+         ${instdir}${prefix}/share/doc/${PKG}-${VER} ;
     /usr/bin/install -m 644 $templist \
-         ${instdir}${prefix}/doc/${PKG3}-${VER} ;
+         ${instdir}${prefix}/share/doc/${PKG3}-${VER} ;
   fi && \
   cat ${srcdir}/CYGWIN-PATCHES/${PKG}.frag \
     ${srcdir}/CYGWIN-PATCHES/README.frag > \
-    ${instdir}${prefix}/doc/Cygwin/${PKG}-${VER}.README && \
+    ${instdir}${prefix}/share/doc/Cygwin/${PKG}-${VER}.README && \
   cat ${srcdir}/CYGWIN-PATCHES/${PKG3}.frag \
     ${srcdir}/CYGWIN-PATCHES/README.frag > \
-    ${instdir}${prefix}/doc/Cygwin/${PKG3}-${VER}.README && \
+    ${instdir}${prefix}/share/doc/Cygwin/${PKG3}-${VER}.README && \
   /usr/bin/install -m 755 ${srcdir}/CYGWIN-PATCHES/postinstall.sh \
-    ${instdir}${sysconfdir}/postinstall/${PKG3}.sh )
-}
+    ${instdir}${sysconfdir}/postinstall/${PKG3}.sh
+)}
+
 strip() {
   (cd ${instdir} && \
   find . -name "*.dll" | xargs strip > /dev/null 2>&1 
   find . -name "*.exe" | xargs strip > /dev/null 2>&1
-  /bin/true )
-}
+  /bin/true
+)}
+
 pkg() {
   (cd ${instdir} && \
   tar cvjf ${bin_pkg} --exclude="${pnoslash}/bin" \
     --exclude="${pnoslash}/include" \
     --exclude="${pnoslash}/lib" \
     --exclude="${snoslash}/postinstall" \
-    --exclude="${pnoslash}/doc/${PKG3}*" \
-    --exclude="${pnoslash}/doc/Cygwin/${PKG3}*" \
+    --exclude="${pnoslash}/share/doc/${PKG3}*" \
+    --exclude="${pnoslash}/share/doc/Cygwin/${PKG3}*" \
     * ;
   tar cvjf ${bin_pkg2} ${pnoslash}/bin/*.dll ;
   tar cvjf ${bin_pkg3} --exclude="${pnoslash}/bin/*.dll" \
@@ -147,26 +165,29 @@ pkg() {
     ${pnoslash}/include \
     ${pnoslash}/lib \
     ${snoslash}/postinstall \
-    ${pnoslash}/doc/${PKG3}* \
-    ${pnoslash}/doc/Cygwin/${PKG3}* )
-}
+    ${pnoslash}/share/doc/${PKG3}* \
+    ${pnoslash}/share/doc/Cygwin/${PKG3}*
+)}
+
 mkpatch() {
   (cd ${srcdir} && \
-  tar xvjf ${src_orig_pkg} ;\
+  tar xv${opt_decomp}f ${src_orig_pkg} ;\
   mv ${PKG}-${VER} ../${PKG}-${VER}-orig && \
   cd ${topdir} && \
   diff -urN -x '.build' -x '.inst' -x '.sinst' \
     ${PKG}-${VER}-orig ${PKG}-${VER} > \
     ${srcinstdir}/${src_patch_name} ; \
-  rm -rf ${PKG}-${VER}-orig )
-}
+  rm -rf ${PKG}-${VER}-orig
+)}
+
 spkg() {
   (mkpatch && \
   cp ${src_orig_pkg} ${srcinstdir}/${src_orig_pkg_name} && \
   cp $0 ${srcinstdir}/`basename $0` && \
   cd ${srcinstdir} && \
-  tar cvjf ${src_pkg} * )
-}
+  tar cvjf ${src_pkg} *
+)}
+
 finish() {
   rm -rf ${srcdir} 
 }
